@@ -1,27 +1,33 @@
+import jax.numpy as jnp
+from jax import grad, jit, random, jacrev, vmap
+from jax.lax import cond, sub
+
 import numpy as np
 import matplotlib.pyplot as plt
-import jax
-from jax.experimental.ode import odeint
-from jax_cosmo.scipy.interpolate import InterpolatedUnivariateSpline
-from tinygp import kernels, GaussianProcess
-import jax.numpy as jnp
-from jax import grad, jit, random
-from functools import partial
-import ticktack
-from astropy.table import Table
-from tqdm import tqdm
-import emcee
-from jax.lax import cond, sub
-from chainconsumer import ChainConsumer
-import scipy
-import seaborn as sns
-from jax import jit, grad, jacrev, vmap
-from jaxns.nested_sampler import NestedSampler
-from jaxns.prior_transforms import PriorChain, UniformPrior
-import os
 from matplotlib.lines import Line2D
 import matplotlib as mpl
 from matplotlib.ticker import MaxNLocator
+import scipy
+
+import ticktack
+
+import diffrax 
+from jax_cosmo.scipy.interpolate import InterpolatedUnivariateSpline
+from tinygp import kernels, GaussianProcess
+
+from functools import partial
+
+from astropy.table import Table
+from tqdm import tqdm
+import emcee
+
+from chainconsumer import ChainConsumer
+
+import seaborn as sns
+# from jaxns.nested_sampler import NestedSampler
+# from jaxns.prior_transforms import PriorChain, UniformPrior
+
+import os
 
 mpl.style.use('seaborn-colorblind')
 
@@ -62,46 +68,46 @@ class CarbonFitter:
         print("Running production...")
         sampler.reset()
         sampler.run_mcmc(p0, production, progress=True);
-        return sampler.flatchain
+        return sampler.get_chain(flat=True)
 
-    def NestedSampler(self, params, likelihood, low_bound=None, high_bound=None, sampler_name='multi_ellipsoid'):
-        """
-        Runs Nested Sampling sampler on the parameter space of some model, subject to some likelihood function.
-        Parameters
-        ----------
-        params : ndarray
-            Initial parameters for NS sampler
-        likelihood : callable
-            Log-likelihood function for params
-        low_bound : ndarray, optional
-            Lower bound of params
-        high_bound : ndarray, optional
-            Upper bound of params
-        sampler_name : str, optional
-            The sampling method for NS sampler. Must be one in: 'multi_ellipsoid', 'slice'. 'multi_ellipsoid' by default.
-        Returns
-        -------
-        ndarray
-            A chain of NS samples
-        """
-
-        @jit
-        def likelihood_function(params, **kwargs):
-            return likelihood(params)
-
-        ndim = params.size
-        if low_bound is not None and high_bound is not None:
-            low_bound = jnp.array(low_bound)
-            high_bound = jnp.array(high_bound)
-        else:
-            low_bound = jnp.array(ndim * None)
-            high_bound = jnp.array(ndim * None)
-        prior_chain = PriorChain().push(UniformPrior('params', low=low_bound, high=high_bound))
-        ns = NestedSampler(likelihood_function, prior_chain, num_live_points=100 * prior_chain.U_ndims,
-                           sampler_name=sampler_name)
-        results = jit(ns)(key=random.PRNGKey(0))
-        # summary(results)
-        return results
+    # def NestedSampler(self, params, likelihood, low_bound=None, high_bound=None, sampler_name='multi_ellipsoid'):
+    #     """
+    #     Runs Nested Sampling sampler on the parameter space of some model, subject to some likelihood function.
+    #     Parameters
+    #     ----------
+    #     params : ndarray
+    #         Initial parameters for NS sampler
+    #     likelihood : callable
+    #         Log-likelihood function for params
+    #     low_bound : ndarray, optional
+    #         Lower bound of params
+    #     high_bound : ndarray, optional
+    #         Upper bound of params
+    #     sampler_name : str, optional
+    #         The sampling method for NS sampler. Must be one in: 'multi_ellipsoid', 'slice'. 'multi_ellipsoid' by default.
+    #     Returns
+    #     -------
+    #     ndarray
+    #         A chain of NS samples
+    #     """
+    #
+    #     @jit
+    #     def likelihood_function(params, **kwargs):
+    #         return likelihood(params)
+    #
+    #     ndim = params.size
+    #     if low_bound is not None and high_bound is not None:
+    #         low_bound = jnp.array(low_bound)
+    #         high_bound = jnp.array(high_bound)
+    #     else:
+    #         low_bound = jnp.array(ndim * None)
+    #         high_bound = jnp.array(ndim * None)
+    #     prior_chain = PriorChain().push(UniformPrior('params', low=low_bound, high=high_bound))
+    #     ns = NestedSampler(likelihood_function, prior_chain, num_live_points=100 * prior_chain.U_ndims,
+    #                        sampler_name=sampler_name)
+    #     results = jit(ns)(key=random.PRNGKey(0))
+    #     # summary(results)
+    #     return results
 
     def chain_summary(self, chain, walkers, figsize=(10, 10), labels=None, plot_dist=False, test_convergence=False,
                       label_font_size=8, tick_font_size=8, mle=False):
@@ -225,7 +231,7 @@ class CarbonFitter:
 
     def plot_multiple_chains(self, chains, walker, figsize=(10, 10), title=None, params_labels=None, labels=None,
                              colors=None, alpha=0.5, linewidths=None, plot_dists=False, label_font_size=12,
-                             tick_font_size=8, max_ticks=10, legend=True):
+                             tick_font_size=8, max_ticks=10, legend=True, truth=None):
         """
        Overplots posterior surfaces from multiple chains.
         Parameters
@@ -277,7 +283,7 @@ class CarbonFitter:
         if plot_dists:
             fig = c.plotter.plot_distributions(figsize=figsize, legend=legend)
         else:
-            fig = c.plotter.plot(figsize=figsize, legend=legend)
+            fig = c.plotter.plot(figsize=figsize, legend=legend, truth=truth)
         plt.suptitle(title)
         plt.tight_layout()
         return fig
@@ -335,7 +341,7 @@ class SingleFitter(CarbonFitter):
     """
 
     def __init__(self, cbm, cbm_model, production_rate_units='atoms/cm^2/s', target_C_14=707., box='Troposphere',
-                 hemisphere='north'):
+                 hemisphere='north',adaptive=True):
         """
         Initializes a SingleFitter Object.
         Parameters
@@ -377,15 +383,21 @@ class SingleFitter(CarbonFitter):
                 if (node.get_hemisphere() == self.hemisphere) & (node.get_name() == self.box):
                     self.box_idx = i
         elif cbm_model == "Miyake17":
-            self.steady_state_production = 1.8
+            # self.steady_state_production = 1.8
+            self.steady_state_production = 1.76
             self.steady_state_y0 = self.cbm.equilibrate(production_rate=self.steady_state_production)
             self.box_idx = 1
         elif cbm_model == "Guttler15":
-            self.steady_state_production = self.cbm.equilibrate(target_C_14=target_C_14)
+            # self.steady_state_production = self.cbm.equilibrate(target_C_14=target_C_14)
+            self.steady_state_production = 1.76
             self.steady_state_y0 = self.cbm.equilibrate(production_rate=self.steady_state_production)
             self.box_idx = 1
 
-    def load_data(self, file_name, oversample=1008, burnin_oversample=1, burnin_time=2000, num_offset=4):
+        self.steady_state_box = self.steady_state_y0[self.box_idx]
+        self.adaptive = adaptive
+
+    def load_data(self, file_name, oversample=1008, burnin_oversample=1, burnin_time=2000, num_offset=4,
+        verbose=False):
         """
         Loads d14c data from a csv file.
         Parameters
@@ -415,14 +427,25 @@ class SingleFitter(CarbonFitter):
         self.mask = jnp.in1d(self.annual, self.time_data)
         self.time_data_fine = jnp.linspace(jnp.min(self.annual), jnp.max(self.annual) + 2,
                                            (self.annual.size + 1) * self.oversample)
-        if self.hemisphere == 'north':
-            self.growth = self.get_growth_vector("april-september")
-        else:
-            self.growth = self.get_growth_vector("october-march")
         try:
             self.growth = self.get_growth_vector(data["growth_season"][0])
+            if verbose:
+                with open('data_log.txt', 'a') as f:
+                    f.write(" Custom Growth Season")
+                    f.write("\n")
         except:
-            pass
+            if self.hemisphere == 'north':
+                self.growth = self.get_growth_vector("april-september")
+                if verbose:
+                    with open('data_log.txt', 'a') as f:
+                        f.write(" Growth april-september")
+                        f.write("\n")
+            else:
+                self.growth = self.get_growth_vector("october-march")
+                if verbose:
+                    with open('data_log.txt', 'a') as f:
+                        f.write(" Growth october-march")
+                        f.write("\n")
 
         # define utils for inverse solver now that we have the growth season
         if jnp.count_nonzero(self.growth) == 12:
@@ -480,35 +503,46 @@ class SingleFitter(CarbonFitter):
         if callable(model):
             self.production = model
             self.production_model = 'custom'
+            self.start_time_index = None
         elif model == "simple_sinusoid":
             self.production = self.simple_sinusoid
             self.production_model = 'simple sinusoid'
-        elif model == "simple_sinusoid_sharp":
-            self.production = self.simple_sinusoid_sharp
-        elif model == "simple_sinusoid_prolonged":
-            self.production = self.simple_sinusoid_prolonged
+            self.start_time_index = 0
+        elif model == "spike_only":
+            self.production = self.spike_only
+            self.production_model = 'spike only'
+            self.adaptive = False
+            self.start_time_index = 0
+        # elif model == "simple_sinusoid_sharp":
+        #     self.production = self.simple_sinusoid_sharp
+        # elif model == "simple_sinusoid_prolonged":
+        #     self.production = self.simple_sinusoid_prolonged
         elif model == "flexible_sinusoid":
             self.production = self.flexible_sinusoid
             self.production_model = 'flexible sinusoid'
+            self.start_time_index = 0
         elif model == "flexible_sinusoid_affine_variant":
             self.production = self.flexible_sinusoid_affine_variant
             self.production_model = 'flexible sinusoid affine variant'
-        elif model == "affine":
-            self.production = self.affine
-            self.production_model = 'affine'
+            self.start_time_index = 1
+        # elif model == "affine":
+        #     self.production = self.affine
+        #     self.production_model = 'affine'
         elif model == "control_points":
             self.control_points_time = jnp.arange(self.start, self.end)
             self.control_points_time_fine = jnp.linspace(self.start, self.end,
                                                          int((self.end - self.start) * self.oversample))
             self.production = self.interp_gp
             self.production_model = 'control points'
+            self.start_time_index = None
         elif model == "inverse_solver":
             self.production = self.interp_IS
             self.production_model = 'inverse solver'
+            self.start_time_index = None
         else:
             raise ValueError(
-                "model is not a callable, or does not take value from: simple_sinusoid, simple_sinusoid_sharp, simple_sinusoid_prolonged, flexible_sinusoid, "
-                "flexible_sinusoid_affine_variant, affine, inverse_solver, control_points")
+                "model is not a callable, or does not take value from: simple_sinusoid, flexible_sinusoid, "
+                "flexible_sinusoid_affine_variant, inverse_solver, control_points")
 
     @partial(jit, static_argnums=(0,))
     def interp_gp(self, tval, *args):
@@ -579,9 +613,9 @@ class SingleFitter(CarbonFitter):
         """
         A simple sinusoid production rate model. Tunable parameters are,
         Start time: start time\n
-        Duration: duration\n
+        log_Duration: log10 duration\n
         Phase: phase of the solar cycle\n
-        Area: total radiocarbon delivered
+        log_Area: log10 total radiocarbon delivered
         Parameters
         ----------
         t : ndarray
@@ -593,21 +627,47 @@ class SingleFitter(CarbonFitter):
         ndarray
             Production rate on t
         """
-        start_time, duration, phase, area = jnp.array(list(args)).reshape(-1)
+        start_time, log_duration, phase, log_area = jnp.array(list(args)).reshape(-1)
+        duration, area = 10**log_duration, 10**log_area
         height = self.super_gaussian(t, start_time, duration, area)
         production = self.steady_state_production + 0.18 * self.steady_state_production * jnp.sin(
             2 * np.pi / 11 * t + phase * 2 * np.pi / 11) + height
         return production
 
+
+    @partial(jit, static_argnums=(0,))
+    def spike_only(self, t, *args):
+        """
+        A simple sinusoid production rate model. Tunable parameters are,
+        Start time: start time\n
+        log_Duration: log10 duration\n
+        log_Area: total radiocarbon delivered
+        Parameters
+        ----------
+        t : ndarray
+            Time sampling
+        args : ndarray
+            Tunable parameters. Must include, start time, duration, phase and area
+        Returns
+        -------
+        ndarray
+            Production rate on t
+        """
+        start_time, log_duration, log_area = jnp.array(list(args)).reshape(-1)
+        duration, area = 10**log_duration, 10**log_area
+        height = self.super_gaussian(t, start_time, duration, area)
+        production = self.steady_state_production + height
+        return production
+
     @partial(jit, static_argnums=(0,))
     def flexible_sinusoid(self, t, *args):
         """
         A flexible sinusoid production rate model. Tunable parameters are,
         Start time: start time\n
-        Duration: duration\n
+        log_Duration: log10 duration\n
         Phase: phase of the solar cycle\n
-        Area: total radiocarbon delivered\n
-        Amplitude: solar amplitude
+        log_Area: log10 total radiocarbon delivered\n
+        log_Amplitude: log10 solar amplitude
         Parameters
         ----------
         t : ndarray
@@ -619,22 +679,26 @@ class SingleFitter(CarbonFitter):
         ndarray
             Production rate on t
         """
-        start_time, duration, phase, area, amplitude = jnp.array(list(args)).reshape(-1)
+        start_time, log_duration, phase, log_area, log_amplitude = jnp.array(list(args)).reshape(-1)
+        duration, area, amplitude = 10**log_duration, 10**log_area, 10**log_amplitude
+
         height = self.super_gaussian(t, start_time, duration, area)
         production = self.steady_state_production + amplitude * self.steady_state_production * jnp.sin(
             2 * np.pi / 11 * t + phase * 2 * np.pi / 11) + height
         return production
 
+
+
     @partial(jit, static_argnums=(0,))
     def flexible_sinusoid_affine_variant(self, t, *args):
         """
         A flexible sinusoid production rate model with a linear gradient. Tunable parameters are,
         Gradient: linear gradient\n
         Start time: start time\n
-        Duration: duration\n
+        log_Duration: log10 duration\n
         Phase: phase of the solar cycle\n
-        Area: total radiocarbon delivered\n
-        Amplitude: solar amplitude
+        log_Area: log10 total radiocarbon delivered\n
+        log_Amplitude: log10 solar amplitude
         Parameters
         ----------
         t : ndarray
@@ -646,7 +710,9 @@ class SingleFitter(CarbonFitter):
         ndarray
             Production rate on t
         """
-        gradient, start_time, duration, phase, area, amplitude = jnp.array(list(args)).reshape(-1)
+        gradient, start_time, log_duration, phase, log_area, log_amplitude = jnp.array(list(args)).reshape(-1)
+        duration, area, amplitude = 10**log_duration, 10**log_area, 10**log_amplitude
+        
         height = self.super_gaussian(t, start_time, duration, area)
         production = self.steady_state_production + gradient * (
                 t - self.start) * (t >= self.start) + amplitude * self.steady_state_production * jnp.sin(
@@ -654,6 +720,7 @@ class SingleFitter(CarbonFitter):
             + phase * 2 * np.pi / 11) + height
         return production
 
+
     # @partial(jit, static_argnums=(0,))
     # def affine(self, t, *args):
     #     gradient, start_time, duration, area = jnp.array(list(args)).reshape(-1)
@@ -695,10 +762,17 @@ class SingleFitter(CarbonFitter):
         ndarray
             Value of each box in the CBM during the production period
         """
+        if self.start_time_index is not None:
+            start, duration = params[self.start_time_index], 10**params[self.start_time_index+1] # assumes they are adjacent - careful in future
+            # also assumes duration is log!
+            step_ts = jnp.linspace(start,start+duration,25)
+        else:
+            step_ts = None
         time_values = jnp.linspace(jnp.min(self.annual), jnp.max(self.annual) + 2,
                                    (self.annual.size + 1) * self.oversample)
         box_values, _ = self.cbm.run(time_values, self.production, y0=y0, args=params,
-                                     steady_state_production=self.steady_state_production)
+                                     steady_state_production=self.steady_state_production,
+                                     adaptive=self.adaptive,step_ts=step_ts)
         return box_values
 
     @partial(jit, static_argnums=(0,))
@@ -714,8 +788,14 @@ class SingleFitter(CarbonFitter):
         ndarray
             Predicted d14c value
         """
-        burnin = self.run_burnin(y0=self.steady_state_y0, params=params)
-        event = self.run_event(y0=burnin[-1, :], params=params)
+        if self.adaptive:
+            burnin = self.run_burnin(y0=self.steady_state_y0, params=params)
+            y0 = burnin[-1, :]
+        else:
+            y0 = self.steady_state_y0
+
+        event = self.run_event(y0=y0, params=params)
+
         binned_data = self.cbm.bin_data(event[:, self.box_idx], self.oversample, self.annual, growth=self.growth)
         d14c = (binned_data - self.steady_state_y0[self.box_idx]) / self.steady_state_y0[self.box_idx] * 1000
         return d14c[self.mask] + self.offset
@@ -733,8 +813,13 @@ class SingleFitter(CarbonFitter):
         ndarray
             Predicted d14c value
         """
-        burnin = self.run_burnin(y0=self.steady_state_y0, params=params)
-        event = self.run_event(y0=burnin[-1, :], params=params)
+        if self.adaptive:
+            burnin = self.run_burnin(y0=self.steady_state_y0, params=params)
+            y0 = burnin[-1, :]
+        else:
+            y0 = self.steady_state_y0
+
+        event = self.run_event(y0=y0, params=params)
         d14c = (event[:, self.box_idx] - self.steady_state_y0[self.box_idx]) / self.steady_state_y0[self.box_idx] * 1000
         return d14c + self.offset
 
@@ -776,6 +861,7 @@ class SingleFitter(CarbonFitter):
         pos = self.log_likelihood(params)
         return lp + pos
 
+
     @partial(jit, static_argnums=(0,))
     def log_likelihood_gp(self, params):
         """
@@ -899,7 +985,7 @@ class SingleFitter(CarbonFitter):
         dash = lambda x: self.dash(x, t_in, data)
 
         # @jit
-        def derivative(y, time):
+        def derivative(time, y, args):
             ans = jnp.matmul(self.cbm.get_matrix(), y)
             prod_coeff = self.cbm.get_production_coefficients()
             production_rate = (dash(time) - ans[self.box_idx]) / prod_coeff[self.box_idx]
@@ -913,9 +999,20 @@ class SingleFitter(CarbonFitter):
         else:
             raise ValueError("Must give either target C-14 or production rate.")
 
-        states = odeint(derivative, steady_state, t_out, atol=1e-15, rtol=1e-15)
+        term = diffrax.ODETerm(derivative)
+        solver = diffrax.Dopri5()
+        saveat = diffrax.SaveAt(ts=t_out)
 
-        flows = jnp.matmul(self.cbm.get_matrix(), states.T)
+        # for fixed time sampling
+        step = 1 / 48. # 4 per month 
+        max_steps = 96*2*jnp.size(t_out)
+        
+        stepsize_controller = diffrax.PIDController(rtol=1e-10, atol=1e-10)
+
+        states = diffrax.diffeqsolve(term, solver, args=(), y0=steady_state, t0 = t_out[0], t1 = t_out[-1],
+            dt0 = step, stepsize_controller=stepsize_controller,saveat=saveat)
+
+        flows = jnp.matmul(self.cbm.get_matrix(), states.ys.T)
         return self._reverse_convert_production_rate((vmap(dash)(t_out) - flows[self.box_idx, :]) /
                                                      self.cbm.get_production_coefficients()[self.box_idx])
 
@@ -959,7 +1056,7 @@ class MultiFitter(CarbonFitter):
     Does parameter fitting, likelihood evaluations, Monte Carlo sampling, plotting and more.
     """
 
-    def __init__(self):
+    def __init__(self,adaptive=True):
         """
         Initializes a MultiFitter object.
         """
@@ -977,6 +1074,7 @@ class MultiFitter(CarbonFitter):
         self.cbm = None
         self.cbm_model = None
         self.box_idx = None
+        self.adaptive=adaptive
 
     def add_SingleFitter(self, sf):
         """
@@ -999,21 +1097,21 @@ class MultiFitter(CarbonFitter):
         if self.production is None:
             self.production = sf.production
             self.production_model = sf.production_model
+            self.start_time_index = sf.start_time_index
         elif self.production_model is not sf.production_model:
             raise ValueError(
                 "production for SingleFitters must be consistent. Got {}, expected {}".format(sf.production_model,
                                                                                               self.production_model))
-
         if self.oversample is None:
             self.oversample = sf.oversample
         elif self.oversample < sf.oversample:
             self.oversample = sf.oversample
 
-        if not self.box_idx:
-            self.box_idx = sf.box_idx
-        elif not self.box_idx == sf.box_idx:
-            raise ValueError(
-                "'box' parameter and 'hemisphere' parameter for SingleFitters must be consistent")
+        # if not self.box_idx:
+        #     self.box_idx = sf.box_idx
+        # elif not self.box_idx == sf.box_idx:
+        #     raise ValueError(
+        #         "'box' parameter and 'hemisphere' parameter for SingleFitters must be consistent")
 
         if self.steady_state_y0 is None:
             self.steady_state_y0 = sf.steady_state_y0
@@ -1028,9 +1126,9 @@ class MultiFitter(CarbonFitter):
             self.cbm_model = sf.cbm_model
             self.cbm = ticktack.load_presaved_model(self.cbm_model, production_rate_units='atoms/cm^2/s')
             self.cbm.compile()
-        elif not self.cbm_model is sf.cbm_model:
-            raise ValueError("cbm model for SingleFitters must be consistent. Got {}, expected {}".format(sf.cbm_model,
-                                                                                                          self.cbm_model))
+        # elif not self.cbm_model is sf.cbm_model:
+        #     raise ValueError("cbm model for SingleFitters must be consistent. Got {}, expected {}".format(sf.cbm_model,
+        #                                                                                                   self.cbm_model))
         self.MultiFitter.append(sf)
 
     def compile(self):
@@ -1102,10 +1200,10 @@ class MultiFitter(CarbonFitter):
         A flexible sinusoid production rate model with a linear gradient. Tunable parameters are,
         Gradient: linear gradient\n
         Start time: start time\n
-        Duration: duration\n
+        log_Duration: log10 duration\n
         Phase: phase of the solar cycle\n
-        Area: total radiocarbon delivered\n
-        Amplitude: solar amplitude
+        log_Area: log10 total radiocarbon delivered\n
+        log_Amplitude: log10 solar amplitude
         Parameters
         ----------
         t : ndarray
@@ -1117,7 +1215,9 @@ class MultiFitter(CarbonFitter):
         ndarray
             Production rate on t
         """
-        gradient, start_time, duration, phase, area, amplitude = jnp.array(list(args)).reshape(-1)
+        gradient, start_time, log_duration, phase, log_area, log_amplitude = jnp.array(list(args)).reshape(-1)
+        duration, area, amplitude = 10**log_duration, 10**log_area, 10**log_amplitude
+
         height = self.super_gaussian(t, start_time, duration, area)
         production = self.steady_state_production + gradient * (
                 t - self.start) * (t >= self.start) + amplitude * self.steady_state_production * jnp.sin(
@@ -1159,10 +1259,17 @@ class MultiFitter(CarbonFitter):
         ndarray
             Value of each box in the CBM during the production period
         """
+        if self.start_time_index is not None:
+            start, duration = params[self.start_time_index], 10**params[self.start_time_index+1] # assumes they are adjacent - careful in future
+            # assumes duration is log space
+            step_ts = jnp.linspace(start,start+duration,25)
+        else:
+            step_ts = None
         time_values = jnp.linspace(jnp.min(self.annual), jnp.max(self.annual) + 2,
                                    (self.annual.size + 1) * self.oversample)
         box_values, _ = self.cbm.run(time_values, self.production, y0=y0, args=params,
-                                     steady_state_production=self.steady_state_production)
+                                     steady_state_production=self.steady_state_production,
+                                     adaptive=self.adaptive)
         return box_values
 
     @partial(jit, static_argnums=(0,))
@@ -1178,8 +1285,12 @@ class MultiFitter(CarbonFitter):
                ndarray
                    Predicted d14c value
                """
-        burnin = self.run_burnin(y0=self.steady_state_y0, params=params)
-        event = self.run_event(y0=burnin[-1, :], params=params)
+        if self.adaptive:
+            burnin = self.run_burnin(y0=self.steady_state_y0, params=params)
+            y0 = burnin[-1, :]
+        else:
+            y0 = self.steady_state_y0
+        event = self.run_event(y0=y0, params=params)
         d14c = (event[:, self.box_idx] - self.steady_state_y0[self.box_idx]) / self.steady_state_y0[self.box_idx] * 1000
         return d14c
 
@@ -1196,12 +1307,18 @@ class MultiFitter(CarbonFitter):
         float
             Log-likelihood
         """
-        burnin = self.run_burnin(y0=self.steady_state_y0, params=params)
-        event = self.run_event(y0=burnin[-1, :], params=params)[:, self.box_idx]
+        if self.adaptive:
+            burnin = self.run_burnin(y0=self.steady_state_y0, params=params) # steady_state_y0 is hem independent
+            y0 = burnin[-1, :]
+        else:
+            y0 = self.steady_state_y0
+
+        event_mat = self.run_event(y0=y0, params=params) # gives a list of boxes
         like = 0
         for sf in self.MultiFitter:
-            binned_data = self.cbm.bin_data(event, self.oversample, self.annual, growth=sf.growth)
-            d14c = (binned_data - self.steady_state_box) / self.steady_state_box * 1000
+            event = event_mat[:, sf.box_idx] # chooses the right box using the information from sf
+            binned_data = sf.cbm.bin_data(event, self.oversample, self.annual, growth=sf.growth) # bins using sf.cbm
+            d14c = (binned_data - sf.steady_state_box) / sf.steady_state_box * 1000 # steady_state_box is hem dependent
             d14c_sf = d14c[sf.multi_mask] + sf.offset
             like += jnp.sum(((sf.d14c_data - d14c_sf) / sf.d14c_data_error) ** 2) * -0.5
         return like
@@ -1245,6 +1362,7 @@ class MultiFitter(CarbonFitter):
         pos = self.multi_likelihood(params)
         return lp + pos
 
+
     def log_joint_likelihood_gp(self, params, low_bounds, up_bounds):
         """
         Computes the log joint likelihood of a set of control-points.
@@ -1351,21 +1469,21 @@ def sample_event(year, mf, sampler='MCMC', production_model='simple_sinusoid', b
         Monte Carlo samples
     """
     if production_model == 'simple_sinusoid':
-        default_params = np.array([year, 1. / 12, 3., 81. / 12])
-        default_low_bounds = jnp.array([year - 5, 1 / 52., 0, 0.])
-        default_up_bounds = jnp.array([year + 5, 5., 11, 15.])
+        default_params = jnp.array([year, np.log10(2. / 12), 3., np.log10(81. / 12)])
+        default_low_bounds = jnp.array([year - 5, np.log10(4 / 52.), 0, -2.])
+        default_up_bounds = jnp.array([year + 5, np.log10(5.), 11, 1.5])
     elif production_model == 'flexible_sinusoid':
-        default_params = np.array([year, 1. / 12, 3., 81. / 12, 0.18])
-        default_low_bounds = jnp.array([year - 5, 1 / 52., 0, 0., 0.])
-        default_up_bounds = jnp.array([year + 5, 5., 11, 15., 2.])
+        default_params = jnp.array([year, np.log10(2. / 12), 3., np.log10(81. / 12), np.log10(0.18)])
+        default_low_bounds = jnp.array([year - 5, np.log10(4 / 52.), 0, -2, -2])
+        default_up_bounds = jnp.array([year + 5, np.log10(5.), 11, 1.5, 1.5])
     elif production_model == 'flexible_sinusoid_affine_variant':
-        default_params = np.array([0, year, 1. / 12, 3., 81. / 12, 0.18])
-        default_low_bounds = jnp.array([-mf.steady_state_production * 0.05 / 5, year - 5, 1 / 52., 0, 0., 0.])
-        default_up_bounds = jnp.array([mf.steady_state_production * 0.05 / 5, year + 5, 5., 11, 15., 0.3])
-    elif production_model == 'affine':
-        default_params = np.array([0, year, 1. / 12, 81. / 12])
-        default_low_bounds = jnp.array([-mf.steady_state_production * 0.05 / 5, year - 5, 1 / 52., 0.])
-        default_up_bounds = jnp.array([mf.steady_state_production * 0.05 / 5, year + 5, 5., 15.])
+        default_params = jnp.array([0, year, np.log10(2. / 12), 3., np.log10(81. / 12), np.log10(0.18)])
+        default_low_bounds = jnp.array([-mf.steady_state_production * 0.05 / 5, year - 5, np.log10(4 / 52.), 0, -2, -2])
+        default_up_bounds = jnp.array([mf.steady_state_production * 0.05 / 5, year + 5, np.log10(5.), 11, 1.5, 1.5])
+    # elif production_model == 'affine':
+    #     default_params = jnp.array([0, year, np.log10(1. / 12), np.log10(81. / 12)])
+    #     default_low_bounds = jnp.array([-mf.steady_state_production * 0.05 / 5, year - 5, np.log10(1 / 52.), -2.])
+    #     default_up_bounds = jnp.array([mf.steady_state_production * 0.05 / 5, year + 5, 2., 1.5])
     elif production_model == 'control_points':
         if sampler == "MCMC":
             default_params = mf.steady_state_production * jnp.ones((len(mf.control_points_time),))
@@ -1400,22 +1518,22 @@ def sample_event(year, mf, sampler='MCMC', production_model='simple_sinusoid', b
                                           burnin=burnin,
                                           production=production,
                                           args=(low_bounds, up_bounds))
-    elif sampler == 'NS':
-        print("Running Nested Sampling...")
-        chain = mf.NestedSampler(params,
-                                 likelihood=mf.multi_likelihood,
-                                 low_bound=low_bounds,
-                                 high_bound=up_bounds
-                                 )
-        print("Done")
+    # elif sampler == 'NS':
+    #     print("Running Nested Sampling...")
+    #     chain = mf.NestedSampler(params,
+    #                              likelihood=mf.multi_likelihood,
+    #                              low_bound=low_bounds,
+    #                              high_bound=up_bounds
+    #                              )
+    #     print("Done")
     else:
         raise ValueError("Invalid sampler value. 'sampler' must be one of the following: MCMC, NS")
     return chain
 
 
-def fit_event(year, event=None, path=None, production_model='simple_sinusoid', cbm_model='Guttler15', box='Troposphere',
+def fit_event(year, path=None, production_model='simple_sinusoid', cbm_model='Guttler15', box='Troposphere',
               hemisphere='north', sampler=None, burnin=500, production=1000, params=None, low_bounds=None,
-              up_bounds=None, mf=None, oversample=1008, burnin_time=2000):
+              up_bounds=None, mf=None, oversample=1008, burnin_time=2000, verbose=False):
     """
     Fits a Miyake event.
 
@@ -1460,24 +1578,22 @@ def fit_event(year, event=None, path=None, production_model='simple_sinusoid', c
     if not mf:
         mf = MultiFitter()
     cbm = ticktack.load_presaved_model(cbm_model, production_rate_units='atoms/cm^2/s')
-    if event:
-        file_names = get_data(event=event)
-        print("Retrieving data...")
-        for file in tqdm(file_names):
-            file_name = 'data/datasets/' + event + '/' + file
+    file_names = get_data(path=path)
+    print("Retrieving data...")
+    for file_name in tqdm(file_names):
+        if 'SH' in file_name:
+            sf = SingleFitter(cbm, cbm_model, box=box, hemisphere='south')
+            if verbose:
+                with open('data_log.txt', 'a') as f:
+                    f.write("%s: Hemisphere South," % file_name)
+        else:
             sf = SingleFitter(cbm, cbm_model, box=box, hemisphere=hemisphere)
-            sf.load_data(os.path.join(os.path.dirname(__file__), file_name), oversample=oversample,
-                         burnin_time=burnin_time)
-            sf.compile_production_model(model=production_model)
-            mf.add_SingleFitter(sf)
-    elif path:
-        file_names = get_data(path=path)
-        print("Retrieving data...")
-        for file_name in tqdm(file_names):
-            sf = SingleFitter(cbm, cbm_model=cbm_model, box=box, hemisphere=hemisphere)
-            sf.load_data(path + '/' + file_name, oversample=oversample, burnin_time=burnin_time)
-            sf.compile_production_model(model=production_model)
-            mf.add_SingleFitter(sf)
+            if verbose:
+                with open('data_log.txt', 'a') as f:
+                    f.write("%s: Hemisphere North," % file_name)
+        sf.load_data(path + '/' + file_name, oversample=oversample, burnin_time=burnin_time, verbose=verbose)
+        sf.compile_production_model(model=production_model)
+        mf.add_SingleFitter(sf)
     mf.compile()
     if not sampler:
         return mf
